@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 November 1989
- * Last Edited:	19 August 2003
+ * Last Edited:	4 May 2004
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 1988-2003 University of Washington.
+ * Copyright 1988-2004 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -28,6 +28,7 @@
 #define MAXAUTHENTICATORS 8	/* maximum number of SASL authenticators */
 				/* maximum number of messages */
 #define MAXMESSAGES (unsigned long) 100000000
+#define MAXLOGINTRIALS 3	/* maximum number of client login attempts */
 
 
 /* These can't be changed without changing code */
@@ -173,6 +174,8 @@
 #define SET_SSLFAILURE (long) 327
 #define GET_NEWSRCCANONHOST (long) 328
 #define SET_NEWSRCCANONHOST (long) 329
+#define GET_KINIT (long) 330
+#define SET_KINIT (long) 331
 
 	/* 4xx: network drivers */
 #define GET_MAXLOGINTRIALS (long) 400
@@ -223,6 +226,12 @@
 #define SET_FETCHLOOKAHEAD (long) 445
 #define GET_NNTPRANGE (long) 446
 #define SET_NNTPRANGE (long) 447
+#define GET_NNTPHIDEPATH (long) 448
+#define SET_NNTPHIDEPATH (long) 449
+#define GET_SENDCOMMAND (long) 450
+#define SET_SENDCOMMAND (long) 451
+#define GET_IDLETIMEOUT (long) 452
+#define SET_IDLETIMEOUT (long) 452
 
 	/* 5xx: local file drivers */
 #define GET_MBXPROTECTION (long) 500
@@ -291,6 +300,8 @@
 #define SET_SNARFMAILBOXNAME (long) 563
 #define GET_SNARFINTERVAL (long) 564
 #define SET_SNARFINTERVAL (long) 565
+#define GET_SNARFPRESERVE (long) 566
+#define SET_SNARFPRESERVE (long) 567
 
 /* Driver flags */
 
@@ -330,7 +341,9 @@
 #define CH_EXPUNGE (long) 45	/* delete elt pointer from list */
 
 
-/* Open options */
+/* Mailbox open options
+ * For compatibility with the past, OP_DEBUG must always be 1.
+ */
 
 #define OP_DEBUG (long) 0x1	/* debug protocol negotiations */
 #define OP_READONLY (long) 0x2	/* read-only open */
@@ -344,6 +357,8 @@
 #define OP_TRYSSL (long) 0x200	/* try SSL first */
 				/* use multiple newsrc files */
 #define OP_MULNEWSRC (long) 0x400
+				/* reserved for application use */
+#define OP_RESERVED (unsigned long) 0xff000000
 
 
 /* Net open options */
@@ -378,6 +393,8 @@
 #define FT_NEEDBODY (long) 0x100/* (internal use) include body structure */
 				/* no fetch lookahead */
 #define FT_NOLOOKAHEAD (long) 0x200
+				/* lookahead in header searching */
+#define FT_SEARCHLOOKAHEAD (long) 0x400
 
 
 /* Flagging options */
@@ -408,6 +425,8 @@
 #define SE_NOLOCAL (long) 0x200	/* no local retry (IMAP only) */
 
 #define SO_NOSERVER SE_NOSERVER	/* compatibility name */
+#define SE_SILLYOK (long) 0x400	/* allow silly searches */
+
 
 /* Status options */
 
@@ -505,6 +524,12 @@
 #define REFSTATUS (long) 8
 #define REFCOPY (long) 9
 #define REFAPPEND (long) 10
+
+
+/* sendcommand_t codes */
+
+				/* expunge response deferred */
+#define SC_EXPUNGEDEFERRED (long) 1
 
 /* Block notification codes */
 
@@ -591,7 +616,7 @@ typedef struct mail_envelope {
   unsigned int imapenvonly : 1;	/* envelope only has IMAP envelope */
   char *remail;			/* remail header if any */
   ADDRESS *return_path;		/* error return address */
-  char *date;			/* message composition date string */
+  unsigned char *date;		/* message composition date string */
   ADDRESS *from;		/* originator address list */
   ADDRESS *sender;		/* sender address list */
   ADDRESS *reply_to;		/* reply address list */
@@ -989,6 +1014,7 @@ typedef struct mail_stream {
   unsigned int perm_draft : 1;	/* permanent Draft flag */
   unsigned int kwd_create : 1;	/* can create new keywords */
   unsigned int uid_nosticky : 1;/* UIDs are not preserved */
+  unsigned int unhealthy : 1;	/* unhealthy protocol negotiations */
   unsigned long perm_user_flags;/* mask of permanent user flags */
   unsigned long gensym;		/* generated tag */
   unsigned long nmsgs;		/* # of associated msgs */
@@ -1110,6 +1136,7 @@ typedef struct send_stream {
   unsigned int debug : 1;	/* stream debug flag */
   unsigned int sensitive : 1;	/* sensitive data in progress */
   unsigned int loser : 1;	/* server is a loser */
+  unsigned int saslcancel : 1;	/* SASL cancelled by protocol */
   union {			/* protocol specific */
     struct {			/* SMTP specific */
       unsigned int ok : 1;	/* supports ESMTP */
@@ -1196,12 +1223,14 @@ typedef void (*smtpverbose_t) (char *buffer);
 typedef void (*imapenvelope_t) (MAILSTREAM *stream,unsigned long msgno,
 				ENVELOPE *env);
 typedef char *(*imapreferral_t) (MAILSTREAM *stream,char *url,long code);
-typedef void (*overview_t) (MAILSTREAM *stream,unsigned long uid,OVERVIEW *ov);
+typedef void (*overview_t) (MAILSTREAM *stream,unsigned long uid,OVERVIEW *ov,
+			    unsigned long msgno);
 typedef unsigned long *(*sorter_t) (MAILSTREAM *stream,char *charset,
 				    SEARCHPGM *spg,SORTPGM *pgm,long flags);
 typedef void (*parseline_t) (ENVELOPE *env,char *hdr,char *data,char *host);
 typedef ADDRESS *(*parsephrase_t) (char *phrase,char *end,char *host);
 typedef void *(*blocknotify_t) (int reason,void *data);
+typedef long (*kinit_t) (char *host,char *reason);
 typedef void (*getacl_t) (MAILSTREAM *stream,char *mailbox,ACLLIST *acl);
 typedef void (*listrights_t) (MAILSTREAM *stream,char *mailbox,char *id,
 			      char *alwaysrights,STRINGLIST *possiblerights);
@@ -1210,6 +1239,7 @@ typedef void (*quota_t) (MAILSTREAM *stream,char *qroot,QUOTALIST *qlist);
 typedef void (*quotaroot_t) (MAILSTREAM *stream,char *mbx,STRINGLIST *qroot);
 typedef void (*sortresults_t) (MAILSTREAM *stream,unsigned long *list,
 			       unsigned long size);
+typedef void (*sendcommand_t) (MAILSTREAM *stream,char *cmd,long flags);
 typedef char *(*newsrcquery_t) (MAILSTREAM *stream,char *mulname,char *name);
 typedef char *(*userprompt_t) (void);
 typedef long (*append_t) (MAILSTREAM *stream,void *data,char **flags,
@@ -1221,8 +1251,7 @@ typedef void (*freestreamsparep_t) (void **sparep);
 typedef void *(*sslstart_t) (void *stream,char *host,unsigned long flags);
 typedef long (*sslcertificatequery_t) (char *reason,char *host,char *cert);
 typedef void (*sslfailure_t) (char *host,char *reason,unsigned long flags);
-
-
+
 /* Globals */
 
 extern char *body_types[];	/* defined body type strings */
@@ -1494,6 +1523,9 @@ MAILSTREAM *mail_stream (MAILHANDLE *handle);
 void mail_fetch_fast (MAILSTREAM *stream,char *sequence,long flags);
 void mail_fetch_flags (MAILSTREAM *stream,char *sequence,long flags);
 void mail_fetch_overview (MAILSTREAM *stream,char *sequence,overview_t ofn);
+void mail_fetch_overview_sequence (MAILSTREAM *stream,char *sequence,
+				   overview_t ofn);
+void mail_fetch_overview_default (MAILSTREAM *stream,overview_t ofn);
 ENVELOPE *mail_fetch_structure (MAILSTREAM *stream,unsigned long msgno,
 				BODY **body,long flags);
 char *mail_fetch_message (MAILSTREAM *stream,unsigned long msgno,
@@ -1539,10 +1571,11 @@ void mail_gc (MAILSTREAM *stream,long gcflags);
 void mail_gc_msg (MESSAGE *msg,long gcflags);
 void mail_gc_body (BODY *body);
 
-BODY *mail_body (MAILSTREAM *stream,unsigned long msgno,char *section);
+BODY *mail_body (MAILSTREAM *stream,unsigned long msgno,
+		 unsigned char *section);
 char *mail_date (char *string,MESSAGECACHE *elt);
 char *mail_cdate (char *string,MESSAGECACHE *elt);
-long mail_parse_date (MESSAGECACHE *elt,char *string);
+long mail_parse_date (MESSAGECACHE *elt,unsigned char *string);
 void mail_exists (MAILSTREAM *stream,unsigned long nmsgs);
 void mail_recent (MAILSTREAM *stream,unsigned long recent);
 void mail_expunged (MAILSTREAM *stream,unsigned long msgno);
@@ -1595,7 +1628,8 @@ THREADNODE *mail_thread_orderedsubject (MAILSTREAM *stream,char *charset,
 THREADNODE *mail_thread_references (MAILSTREAM *stream,char *charset,
 				    SEARCHPGM *spg,long flags,
 				    sorter_t sorter);
-void mail_thread_loadcache (MAILSTREAM *stream,unsigned long uid,OVERVIEW *ov);
+void mail_thread_loadcache (MAILSTREAM *stream,unsigned long uid,OVERVIEW *ov,
+			    unsigned long msgno);
 char *mail_thread_parse_msgid (char *s,char **ss);
 STRINGLIST *mail_thread_parse_references (char *s,long flag);
 long mail_thread_check_child (container_t mother,container_t daughter);
@@ -1604,8 +1638,8 @@ container_t mail_thread_prune_dummy_work (container_t msg,container_t ane);
 THREADNODE *mail_thread_c2node (MAILSTREAM *stream,container_t con,long flags);
 THREADNODE *mail_thread_sort (THREADNODE *thr,THREADNODE **tc);
 int mail_thread_compare_date (const void *a1,const void *a2);
-long mail_sequence (MAILSTREAM *stream,char *sequence);
-long mail_uid_sequence (MAILSTREAM *stream,char *sequence);
+long mail_sequence (MAILSTREAM *stream,unsigned char *sequence);
+long mail_uid_sequence (MAILSTREAM *stream,unsigned char *sequence);
 long mail_parse_flags (MAILSTREAM *stream,char *flag,unsigned long *uf);
 long mail_usable_network_stream (MAILSTREAM *stream,char *name);
 
