@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	1 November 1990
- * Last Edited:	2 December 2002
+ * Last Edited:	27 April 2004
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2002 University of Washington.
+ * Copyright 1988-2004 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -57,7 +57,7 @@ extern int errno;		/* just in case */
 
 /* Global storage */
 
-char *version = "2002.81";	/* server version */
+char *version = "2004.87";	/* server version */
 short state = AUTHORIZATION;	/* server state */
 short critical = NIL;		/* non-zero if in critical code */
 MAILSTREAM *stream = NIL;	/* mailbox stream */
@@ -117,12 +117,11 @@ int main (int argc,char *argv[])
   /* There are reports of POP3 clients which get upset if anything appears
    * between the "+OK" and the "POP3" in the greeting.
    */
-  PSOUT ("+OK POP3");
+  PSOUT ("+OK POP3 ");
   if (!challenge[0]) {		/* if no MD5 enable, output host name */
-    PBOUT (' ');
     PSOUT (tcp_serverhost ());
+    PBOUT (' ');
   }
-  PSOUT (" v");
   PSOUT (version);
   PSOUT (" server ready");
   if (challenge[0]) {		/* if MD5 enable, output challenge here */
@@ -172,26 +171,19 @@ int main (int argc,char *argv[])
 	PSOUT ("TOP\015\012LOGIN-DELAY 180\015\012UIDL\015\012");
 	if (s = ssl_start_tls (NIL)) fs_give ((void *) &s);
 	else PSOUT ("STLS\015\012");
-	if (mail_parameters (NIL,GET_DISABLEPLAINTEXT,NIL)) {
-	  PSOUT ("SASL");	/* display secure server authenticators */
-	  for (auth = mail_lookup_auth (1); auth; auth = auth->next)
-	    if (auth->server) {
-	      if (auth->flags & AU_SECURE) {
-		PBOUT (' ');
-		PSOUT (auth->name);
-	      }
+	if (i = !mail_parameters (NIL,GET_DISABLEPLAINTEXT,NIL))
+	  PSOUT ("USER\015\012");
+				/* display secure server authenticators */
+	for (auth = mail_lookup_auth (1), s = "SASL"; auth; auth = auth->next)
+	  if (auth->server && (i || (auth->flags & AU_SECURE))) {
+	    if (s) {
+	      PSOUT (s);
+	      s = NIL;
 	    }
-	}
-	else {			/* display all authentication means */
-	  PSOUT ("USER\015\012SASL");
-	  for (auth = mail_lookup_auth (1); auth; auth = auth->next)
-	    if (auth->server) {
-	      PBOUT (' ');
-	      PSOUT (auth->name);
-	    }
-	}
-	CRLF;
-	PSOUT (".\015\012");
+	    PBOUT (' ');
+	    PSOUT (auth->name);
+	  }
+	PSOUT (s ? ".\015\012" : "\015\012.\015\012");
       }
 
       else switch (state) {	/* else dispatch based on state */
@@ -218,18 +210,9 @@ int main (int argc,char *argv[])
 	  else {
 	    AUTHENTICATOR *auth;
 	    PSOUT ("+OK Supported authentication mechanisms:\015\012");
-	    if (mail_parameters (NIL,GET_DISABLEPLAINTEXT,NIL)) {
-	      for (auth = mail_lookup_auth (1); auth; auth = auth->next)
-		if (auth->server) {
-		  if (auth->flags & AU_SECURE) {
-		    PSOUT (auth->name);
-		    CRLF;
-		  }
-		}
-	    }
-				/* display all authentication means */
-	    else for (auth = mail_lookup_auth (1); auth; auth = auth->next)
-	      if (auth->server) {
+	    i = !mail_parameters (NIL,GET_DISABLEPLAINTEXT,NIL);
+	    for (auth = mail_lookup_auth (1); auth; auth = auth->next)
+	      if (auth->server && (i || (auth->flags & AU_SECURE))) {
 		PSOUT (auth->name);
 		CRLF;
 	      }
@@ -403,8 +386,9 @@ int main (int argc,char *argv[])
 	  if (t && *t && (i =strtoul (t,&s,10)) && (i <= nmsgs) &&
 	      (msg[i] > 0)) {
 				/* skip whitespace */
-	    while (isspace (*s)) s++;
-	    if (isdigit (*s)) {	/* make sure line count argument good */
+	    while (*s == ' ') s++;
+				/* make sure line count argument good */
+	    if ((*s >= '0') && (*s <= '9')) {
 	      MESSAGECACHE *elt = mail_elt (stream,msg[i]);
 	      j = strtoul (s,NIL,10);
 				/* update highest message accessed */
@@ -463,7 +447,8 @@ int main (int argc,char *argv[])
 	       tcp_clienthost ());
   PSOUT (sayonara);		/* "now it's time to say sayonara..." */
   PFLUSH ();			/* make sure output finished */
-  return 0;			/* all done */
+  exit (0);			/* all done */
+  return 0;			/* stupid compilers */
 }
 
 /* Clock interrupt
