@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	2 February 1994
- * Last Edited:	19 December 2002
+ * Last Edited:	1 May 2004
  *
  * The IMAP tools software provided in this Distribution is
- * Copyright 2002 by the University of Washington
+ * Copyright 1988-2004 by the University of Washington
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -26,12 +26,12 @@ extern int errno;		/* just in case */
 #include "osdep.h"
 #include "misc.h"
 #include "linkage.h"
-char *getpass ();		/* let's all do the cretin hop */
 
 /* Globals */
 
 int debugp = NIL;		/* flag saying debug */
 int verbosep = NIL;		/* flag saying verbose */
+int rwcopyp = NIL;		/* flag saying readwrite copy (for POP) */
 int critical = NIL;		/* flag saying in critical code */
 int trycreate = NIL;		/* [TRYCREATE] seen */
 char *suffix = NIL;		/* suffer merge mode suffix text */
@@ -145,7 +145,9 @@ int main (int argc,char *argv[])
 {
   MAILSTREAM *source = NIL;
   MAILSTREAM *dest = NIL;
+  SEARCHPGM *criteria;
   char c,*s,**args,*dp,*t,*t1,tmp[MAILTMPLEN],mbx[MAILTMPLEN];
+  unsigned long m,len,curlen,start,last;
   int nargs,i;
   int merge = NIL;
   int ret = 1;
@@ -158,6 +160,7 @@ int main (int argc,char *argv[])
     if (*(s = *args) == '-') {	/* parse switches */
       if (!strcmp (s,"-debug") || !strcmp (s,"-d")) debugp = T;
       else if (!strcmp (s,"-verbose") || !strcmp (s,"-v")) verbosep = T;
+      else if (!strcmp (s,"-rwcopy") || !strcmp (s,"-rw")) rwcopyp = T;
       else if ((nargs > 1) && (!strcmp (s,"-merge") || !strcmp (s,"-m"))) {
 	args++,nargs--;		/* advance to next argument */
 	if (!strcmp (s = *args,"prompt")) merge = mPROMPT;
@@ -168,12 +171,12 @@ int main (int argc,char *argv[])
 	}
 	else {
 	  printf ("unknown merge option: %s\n",s);
-	  return ret;
+	  exit (ret);
 	}
       }
       else {
 	printf ("unknown switch: %s\n",s);
-	return ret;
+	exit (ret);
       }
     }
     else if (!cmd) cmd = s;	/* first non-switch is command */
@@ -181,32 +184,48 @@ int main (int argc,char *argv[])
     else if (!dst) dst = s;	/* third non-switch is destination */
     else {
       printf ("unknown argument: %s\n",s);
-      return ret;
+      exit (ret);
     }
   }
   if (!cmd) cmd = "";		/* prevent SEGV */
 
   if (!strcmp (cmd,"check")) {	/* check for new messages */
     if (!src) src = "INBOX";
-    if (dst || merge)
+    if (dst || merge || rwcopyp)
       printf ("usage: %s check [-debug] [-verbose] [mailbox]\n",pgm);
     else if (mail_status (source = (*src == '{') ?
 			  mail_open (NIL,src,OP_HALFOPEN |
 				     (debugp ? OP_DEBUG : NIL)) : NIL,
-			  src,SA_MESSAGES | SA_RECENT)) ret = 0;
+			  src,SA_MESSAGES | SA_RECENT | SA_UNSEEN)) ret = 0;
   }
   else if (!strcmp (cmd,"create")) {
-    if (!src || dst || merge)
+    if (!src || dst || merge || rwcopyp)
       printf ("usage: %s create [-debug] [-verbose] mailbox\n",pgm);
     else if (mail_create (source = (*src == '{') ?
 			  mail_open (NIL,src,OP_HALFOPEN |
 				     (debugp ? OP_DEBUG : NIL)) : NIL,src))
       ret = 0;
   }
+  else if (!strcmp (cmd,"delete")) {
+    if (!src || dst || merge || rwcopyp)
+      printf ("usage: %s delete [-debug] [-verbose] mailbox\n",pgm);
+    else if (mail_delete (source = (*src == '{') ?
+			  mail_open (NIL,src,OP_HALFOPEN |
+				     (debugp ? OP_DEBUG : NIL)) : NIL,src))
+      ret = 0;
+  }
+  else if (i = !strcmp (cmd,"rename")) {
+    if (!src || !dst || merge || rwcopyp)
+      printf ("usage: %s %s [-debug] [-verbose] source destination\n",pgm,cmd);
+    else if (mail_rename (source = (*src == '{') ?
+			  mail_open (NIL,src,OP_HALFOPEN |
+				     (debugp ? OP_DEBUG : NIL)) : NIL,src,dst))
+      ret = 0;
+  }
   else if ((i = !strcmp (cmd,"move")) || !strcmp (cmd,"copy")) {
     if (!src || !dst || merge)
       printf ("usage: %s %s [-debug] [-verbose] source destination\n",pgm,cmd);
-    else if (source = mail_open (NIL,src,(i ? NIL : OP_READONLY) |
+    else if (source = mail_open (NIL,src,((i || rwcopyp) ? NIL : OP_READONLY) |
 				 (debugp ? OP_DEBUG : NIL))) {
       dest = NIL;		/* open destination stream if network */
       if ((*dst != '{') || (dest = mail_open (NIL,dst,OP_HALFOPEN |
@@ -218,7 +237,7 @@ int main (int argc,char *argv[])
   else if ((i = !strcmp (cmd,"appenddelete")) || !strcmp (cmd,"append")) {
     if (!src || !dst || merge)
       printf ("usage: %s %s [-debug] [-verbose] source destination\n",pgm,cmd);
-    else if (source = mail_open (NIL,src,(i ? NIL : OP_READONLY) |
+    else if (source = mail_open (NIL,src,((i || rwcopyp) ? NIL : OP_READONLY) |
 				 (debugp ? OP_DEBUG : NIL))) {
       dest = NIL;		/* open destination stream if network */
       if ((*dst != '{') || (dest = mail_open (NIL,dst,OP_HALFOPEN |
@@ -227,6 +246,47 @@ int main (int argc,char *argv[])
       }
     }
   }
+
+  else if (!strcmp (cmd,"prune")) {
+    if (!src || !dst || merge || rwcopyp || !(criteria = mail_criteria (dst)))
+      printf ("usage: %s prune [-debug] [-verbose] mailbox search_criteria\n",
+	      pgm);
+    else if ((source = mail_open (NIL,src,(debugp ? OP_DEBUG : NIL))) &&
+	     mail_search_full (source,NIL,criteria,SE_FREE)) {
+      for (m = 1, s = t = NIL, len = start = last = 0; m <= source->nmsgs; m++)
+	if (mail_elt (source,m)->searched) {
+	  if (s) {		/* continuing a range? */
+	    if (m == last + 1) last = m;
+	    else {		/* no, end of previous range? */
+	      if (last != start) sprintf (t,":%lu,%lu",last,m);
+				/* no, just this message */
+	      else sprintf (t,",%lu",m);
+				/* running out of space? */
+	      if ((len - (curlen = (t += strlen (t)) - s)) < 20) {
+		fs_resize ((void **) &s,len += MAILTMPLEN);
+		t = s + curlen;	/* relocate current pointer */
+	      }
+	    }
+	  }
+	  else {		/* first time, start new buffer */
+	    s = (char *) fs_get (len = MAILTMPLEN);
+	    sprintf (s,"%lu",start = last = m);
+	    t = s + strlen (s);	/* end of buffer */
+	  }
+	}
+				/* finish last range if necessary */
+      if (last != start) sprintf (t,":%lu",last);
+      if (s) {			/* delete/expunge any matching messages */
+	mail_flag (source,s,"\\Deleted",ST_SET);
+	m = source->nmsgs;	/* get number of messages before purge */
+	mail_expunge (source);
+	printf ("%lu message(s) purged\n",m - source->nmsgs);
+	fs_give ((void **) &s);	/* flush buffer */
+      }
+      else puts ("No matching messages, so nothing purged");
+      source = mail_close (source);
+    }
+  }
 
   else if (!strcmp (cmd,"transfer")) {
     if (!src || !dst)
@@ -264,7 +324,8 @@ int main (int argc,char *argv[])
 	for (t = mbx,t1 = dest ? dest->mailbox : "",c = NIL; (c != '}') && *t1;
 	     *t++ = c= *t1++);
 	for (t1 = dp; *t1; *t++ = *t1++);
-	t1 = source ? (strchr (tmp+1,'}') + 1) : tmp;
+				/* point to name without delim or netspec */
+	t1 = source ? (strchr (tmp+1,'}') + 1) : tmp + 1;
 				/* src and mbx have different delimiters? */
 	if (ddelim && (ddelim != tmp[0]))
 	  while (c = *t1++) {	/* swap delimiters then */
@@ -279,20 +340,30 @@ int main (int argc,char *argv[])
 	  printf ("Copying %s\n  => %s\n",tmp+1,mbx);
 	  fflush (stdout);
 	}
-	if (source = mail_open (source,tmp+1,OP_READONLY))
+	if (source = mail_open (source,tmp+1,rwcopyp ? NIL : OP_READONLY)) {
 	  ret = mbxcopy (source,dest,mbx,T,NIL,merge);
+	  if (source->dtb->flags & DR_LOCAL) source = mail_close (source);
+	}
 	else printf ("can't open source mailbox %s\n",tmp+1);
       }
     }
   }
+
   else {
-    printf ("usage: %s check [-debug] [-verbose] mailbox\n",pgm);
+    printf ("usage: %s check [-debug] [-verbose] [mailbox]\n",pgm);
     puts   ("        ;; report number of messages and new messages");
     printf ("       %s create [-debug] [-verbose] new_mailbox\n",pgm);
     puts   ("        ;; create new mailbox");
+    printf ("       %s delete [-debug] [-verbose] mailbox\n",pgm);
+    puts   ("        ;; delete existing mailbox");
+    printf ("       %s rename [-debug] [-verbose] source destination\n",pgm);
+    puts   ("        ;; rename mailbox to a new name");
     printf ("       %s (copy | move) [-debug] [-verbose] old_mailbox new_mailbox\n",pgm);
     puts   ("        ;; create new mailbox and copy/move messages");
     printf ("       %s (append | appenddelete) [-debug] [-verbose] source destination\n",pgm);
+    puts   ("        ;; prune mailbox of messages matching criteria");
+    printf ("       %s prune [-debug] [-verbose] mailbox search_criteria\n",
+	    pgm);
     puts   ("        ;; copy/move messages to existing mailbox");
     printf ("       %s transfer [-debug] [-verbose] [-merge mode] source destination\n",pgm);
     puts   ("        ;; make copy of source hierarchy to destination");
@@ -301,7 +372,8 @@ int main (int argc,char *argv[])
 				/* close streams */
   if (source) mail_close (source);
   if (dest) mail_close (dest);
-  return ret;
+  exit (ret);
+  return ret;			/* stupid compilers */
 }
 
 /* Copy mailbox
@@ -398,6 +470,7 @@ long mm_append (MAILSTREAM *stream,void *data,char **flags,char **date,
   unsigned long u;
   MESSAGECACHE *elt;
   APPENDPACKAGE *ap = (APPENDPACKAGE *) data;
+  *flags = *date = NIL;		/* assume no flags or date */
   if (ap->flags) fs_give ((void **) &ap->flags);
   if (ap->date) fs_give ((void **) &ap->date);
   mail_gc (ap->stream,GC_TEXTS);
@@ -437,7 +510,7 @@ long mm_append (MAILSTREAM *stream,void *data,char **flags,char **date,
 
 void mm_searched (MAILSTREAM *stream,unsigned long msgno)
 {
-  fatal ("impossible mm_searched() call");
+				/* dummy routine */
 }
 
 
@@ -511,7 +584,8 @@ void mm_lsub (MAILSTREAM *stream,int delimiter,char *name,long attributes)
 
 void mm_status (MAILSTREAM *stream,char *mailbox,MAILSTATUS *status)
 {
-  if (status->recent) printf ("%lu new message(s),",status->recent);
+  if (status->recent || status->unseen)
+    printf ("%lu new message(s) (%lu unseen),",status->recent,status->unseen);
   else fputs ("No new messages,",stdout);
   printf (" %lu total in %s\n",status->messages,mailbox);
 }
