@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	22 May 1990
- * Last Edited:	21 October 2002
+ * Last Edited:	4 May 2004
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2002 University of Washington.
+ * Copyright 1988-2004 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -24,6 +24,11 @@
  * The atime is the last read time of the file.
  * The mtime is the last flags update time of the file.
  * The ctime is the last write time of the file.
+ *
+ *				TEXT SIZE SEMANTICS
+ *
+ * Most of the text sizes are in internal (LF-only) form, except for the
+ * msg.text size.  Beware.
  */
 
 #include <stdio.h>
@@ -36,9 +41,67 @@ extern int errno;		/* just in case */
 #include <time.h>
 #include <sys/stat.h>
 #include <sys/utime.h>
-#include "tenexnt.h"
 #include "misc.h"
 #include "dummy.h"
+
+/* TENEX I/O stream local data */
+	
+typedef struct tenex_local {
+  unsigned int shouldcheck: 1;	/* if ping should do a check instead */
+  unsigned int mustcheck: 1;	/* if ping must do a check instead */
+  int fd;			/* file descriptor for I/O */
+  off_t filesize;		/* file size parsed */
+  time_t filetime;		/* last file time */
+  time_t lastsnarf;		/* local snarf time */
+  unsigned char *buf;		/* temporary buffer */
+  unsigned long buflen;		/* current size of temporary buffer */
+  unsigned long uid;		/* current text uid */
+  SIZEDTEXT text;		/* current text */
+} TENEXLOCAL;
+
+
+/* Convenient access to local data */
+
+#define LOCAL ((TENEXLOCAL *) stream->local)
+
+
+/* Function prototypes */
+
+DRIVER *tenex_valid (char *name);
+int tenex_isvalid (char *name,char *tmp);
+void *tenex_parameters (long function,void *value);
+void tenex_scan (MAILSTREAM *stream,char *ref,char *pat,char *contents);
+void tenex_list (MAILSTREAM *stream,char *ref,char *pat);
+void tenex_lsub (MAILSTREAM *stream,char *ref,char *pat);
+long tenex_create (MAILSTREAM *stream,char *mailbox);
+long tenex_delete (MAILSTREAM *stream,char *mailbox);
+long tenex_rename (MAILSTREAM *stream,char *old,char *newname);
+long tenex_status (MAILSTREAM *stream,char *mbx,long flags);
+MAILSTREAM *tenex_open (MAILSTREAM *stream);
+void tenex_close (MAILSTREAM *stream,long options);
+void tenex_fast (MAILSTREAM *stream,char *sequence,long flags);
+void tenex_flags (MAILSTREAM *stream,char *sequence,long flags);
+char *tenex_header (MAILSTREAM *stream,unsigned long msgno,
+		    unsigned long *length,long flags);
+long tenex_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags);
+void tenex_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags);
+void tenex_flagmsg (MAILSTREAM *stream,MESSAGECACHE *elt);
+long tenex_ping (MAILSTREAM *stream);
+void tenex_check (MAILSTREAM *stream);
+void tenex_snarf (MAILSTREAM *stream);
+void tenex_expunge (MAILSTREAM *stream);
+long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options);
+long tenex_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data);
+
+unsigned long tenex_size (MAILSTREAM *stream,unsigned long m);
+long tenex_parse (MAILSTREAM *stream);
+MESSAGECACHE *tenex_elt (MAILSTREAM *stream,unsigned long msgno);
+void tenex_read_flags (MAILSTREAM *stream,MESSAGECACHE *elt);
+void tenex_update_status (MAILSTREAM *stream,unsigned long msgno,
+			  long syncflag);
+unsigned long tenex_hdrpos (MAILSTREAM *stream,unsigned long msgno,
+			    unsigned long *size);
+
 
 /* Tenex mail routines */
 
@@ -113,7 +176,7 @@ int tenex_isvalid (char *name,char *tmp)
   struct utimbuf times;
   errno = EINVAL;		/* assume invalid argument */
 				/* if file, get its status */
-  if ((s = mailboxfile (file,name)) && !stat (s,&sbuf) &&
+  if ((s = dummy_file (file,name)) && !stat (s,&sbuf) &&
       ((sbuf.st_mode & S_IFMT) == S_IFREG)) {
     if (!sbuf.st_size)errno = 0;/* empty file */
     else if ((fd = open (file,O_BINARY|O_RDONLY,NIL)) >= 0) {
@@ -124,14 +187,17 @@ int tenex_isvalid (char *name,char *tmp)
 				/* must begin with dd-mmm-yy" */
 	ret = (((tmp[2] == '-' && tmp[6] == '-') ||
 		(tmp[1] == '-' && tmp[5] == '-')) &&
-	       (s = strchr (tmp+20,',')) && strchr (s+2,';')) ? T : NIL;
+	       (s = strchr (tmp+18,',')) && strchr (s+2,';')) ? T : NIL;
       }
       else errno = -1;		/* bogus format */
       close (fd);		/* close the file */
+				/* \Marked status? */
+      if (sbuf.st_ctime > sbuf.st_atime) {
 				/* preserve atime and mtime */
-      times.actime = sbuf.st_atime;
-      times.modtime = sbuf.st_mtime;
-      utime (file,&times);	/* set the times */
+	times.actime = sbuf.st_atime;
+	times.modtime = sbuf.st_mtime;
+	utime (file,&times);	/* set the times */
+      }
     }
   }
 				/* in case INBOX but not tenex format */
@@ -223,18 +289,26 @@ long tenex_delete (MAILSTREAM *stream,char *mailbox)
 
 long tenex_rename (MAILSTREAM *stream,char *old,char *newname)
 {
-  long ret = T;
+  long ret = LONGT;
   char c,*s,tmp[MAILTMPLEN],file[MAILTMPLEN],lock[MAILTMPLEN];
-  int ld;
-  int fd = open (mailboxfile (file,old),O_BINARY|O_RDWR,NIL);
+  int fd,ld;
   struct stat sbuf;
-  if (fd < 0) {			/* open mailbox */
+  if (!dummy_file (file,old) ||
+      (newname && !((s = mailboxfile (tmp,newname)) && *s))) {
+    sprintf (tmp,newname ?
+	     "Can't rename mailbox %.80s to %.80s: invalid name" :
+	     "Can't delete mailbox %.80s: invalid name",
+	     old,newname);
+    mm_log (tmp,ERROR);
+    return NIL;
+  }
+  else if ((fd = open (file,O_BINARY|O_RDWR,NIL)) < 0) {
     sprintf (tmp,"Can't open mailbox %.80s: %s",old,strerror (errno));
     mm_log (tmp,ERROR);
     return NIL;
   }
 				/* get exclusive parse/append permission */
-  if ((ld = lockname (lock,old,LOCK_EX)) < 0) {
+  if ((ld = lockname (lock,file,LOCK_EX)) < 0) {
     mm_log ("Unable to lock rename mailbox",ERROR);
     return NIL;
   }
@@ -246,16 +320,11 @@ long tenex_rename (MAILSTREAM *stream,char *old,char *newname)
     unlockfd (ld,lock);		/* release exclusive parse/append permission */
     return NIL;
   }
+
   if (newname) {		/* want rename? */
-    if (!((s = mailboxfile (tmp,newname)) && *s) ||
-	((s = strrchr (s,'\\')) && !s[1])) {
-      sprintf (tmp,"Can't rename mailbox %.80s to %.80s: invalid name",
-	       old,newname);
-      mm_log (tmp,ERROR);
-      ret = NIL;		/* set failure */
-    }
 				/* found superior to destination name? */
-    else if (s && (s != tmp) && ((tmp[1] != ':') || (s != tmp + 2))) {
+    if ((s = strrchr (tmp,'\\')) && (s != tmp) &&
+	((tmp[1] != ':') || (s != tmp + 2))) {
       c = s[1];			/* remember character after delimiter */
       *s = s[1] = '\0';		/* tie off name at delimiter */
 				/* name doesn't exist, create it */
@@ -301,9 +370,14 @@ MAILSTREAM *tenex_open (MAILSTREAM *stream)
 				/* return prototype for OP_PROTOTYPE call */
   if (!stream) return &tenexproto;
   if (stream->local) fatal ("tenex recycle stream");
+				/* canonicalize the mailbox name */
+  if (!dummy_file (tmp,stream->mailbox)) {
+    sprintf (tmp,"Can't open - invalid name: %.80s",stream->mailbox);
+    mm_log (tmp,ERROR);
+  }
   if (stream->rdonly ||
-      (fd = open (mailboxfile (tmp,stream->mailbox),O_BINARY|O_RDWR,NIL)) < 0){
-    if ((fd=open(mailboxfile(tmp,stream->mailbox),O_BINARY|O_RDONLY,NIL)) < 0){
+      (fd = open (tmp,O_BINARY|O_RDWR,NIL)) < 0) {
+    if ((fd = open (tmp,O_BINARY|O_RDONLY,NIL)) < 0) {
       sprintf (tmp,"Can't open mailbox: %.80s",strerror (errno));
       mm_log (tmp,ERROR);
       return NIL;
@@ -317,6 +391,8 @@ MAILSTREAM *tenex_open (MAILSTREAM *stream)
   LOCAL->fd = fd;		/* bind the file */
   LOCAL->buf = (char *) fs_get (MAXMESSAGESIZE + 1);
   LOCAL->buflen = MAXMESSAGESIZE;
+  LOCAL->text.data = (unsigned char *)
+    fs_get ((LOCAL->text.size = MAXMESSAGESIZE) + 1);
 				/* note if an INBOX or not */
   stream->inbox = !compare_cstring (stream->mailbox,"INBOX");
   fs_give ((void **) &stream->mailbox);
@@ -361,6 +437,7 @@ void tenex_close (MAILSTREAM *stream,long options)
     close (LOCAL->fd);		/* close the local file */
 				/* free local text buffer */
     if (LOCAL->buf) fs_give ((void **) &LOCAL->buf);
+    if (LOCAL->text.data) fs_give ((void **) &LOCAL->text.data);
 				/* nuke the local data */
     fs_give ((void **) &stream->local);
     stream->dtb = NIL;		/* log out the DTB */
@@ -466,28 +543,40 @@ long tenex_text (MAILSTREAM *stream,unsigned long msgno,STRING *bs,long flags)
     tenex_update_status (stream,msgno,T);
     mm_flags (stream,msgno);
   }
-				/* find header position */
-  i = tenex_hdrpos (stream,msgno,&j);
-				/* go to text position */
-  lseek (LOCAL->fd,i + j,L_SET);
   if (flags & FT_INTERNAL) {	/* if internal representation wanted */
+				/* find header position */
+    i = tenex_hdrpos (stream,msgno,&j);
     if (i > LOCAL->buflen) {	/* resize if not enough space */
       fs_give ((void **) &LOCAL->buf);
       LOCAL->buf = (char *) fs_get (LOCAL->buflen = i + 1);
     }
+				/* go to text position */
+    lseek (LOCAL->fd,i + j,L_SET);
 				/* slurp the data */
     if (read (LOCAL->fd,LOCAL->buf,i) != (long) i) return NIL;
+				/* set up stringstruct for internal */
+    INIT (bs,mail_string,LOCAL->buf,i);
   }
-  else {			/* get readin buffer */
-    s = (char *) fs_get ((i = tenex_size (stream,msgno) - j) + 1);
-    s[i] = '\0';		/* tie off string */
-    read (LOCAL->fd,s,i);	/* slurp the data */
+  else {			/* normal form, previous text cached? */
+    if (elt->private.uid == LOCAL->uid)
+      i = elt->private.msg.text.text.size;
+    else {			/* not cached, cache it now */
+      LOCAL->uid = elt->private.uid;
+				/* find header position */
+      i = tenex_hdrpos (stream,msgno,&j);
+				/* go to text position */
+      lseek (LOCAL->fd,i + j,L_SET);
+      s = (char *) fs_get ((i = tenex_size (stream,msgno) - j) + 1);
+      s[i] = '\0';		/* tie off string */
+      read (LOCAL->fd,s,i);	/* slurp the data */
 				/* make CRLF copy of string */
-    i = unix_crlfcpy (&LOCAL->buf,&LOCAL->buflen,s,i);
-    fs_give ((void **) &s);	/* free readin buffer */
-  }
+      i = elt->private.msg.text.text.size =
+	strcrlfcpy (&LOCAL->text.data,&LOCAL->text.size,s,i);
+      fs_give ((void **) &s);	/* free readin buffer */
+    }
 				/* set up stringstruct */
-  INIT (bs,mail_string,LOCAL->buf,i);
+    INIT (bs,mail_string,LOCAL->text.data,i);
+  }
   return T;			/* success */
 }
 
@@ -506,7 +595,7 @@ void tenex_flag (MAILSTREAM *stream,char *sequence,char *flag,long flags)
     fsync (LOCAL->fd);
     fstat (LOCAL->fd,&sbuf);	/* get current write time */
     times.modtime = LOCAL->filetime = sbuf.st_mtime;
-    times.actime = time (0);		/* make sure read comes after all that */
+    times.actime = time (0);	/* make sure read comes after all that */
     utime (stream->mailbox,&times);
   }
 }
@@ -668,7 +757,7 @@ void tenex_expunge (MAILSTREAM *stream)
   fsync (LOCAL->fd);		/* force disk update */
   fstat (LOCAL->fd,&sbuf);	/* get new write time */
   times.modtime = LOCAL->filetime = sbuf.st_mtime;
-  times.actime = time (0);		/* reset atime to now */
+  times.actime = time (0);	/* reset atime to now */
   utime (stream->mailbox,&times);
   mm_nocritical (stream);	/* release critical */
 				/* notify upper level of new mailbox size */
@@ -718,7 +807,7 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
   if (!((options & CP_UID) ? mail_uid_sequence (stream,sequence) :
 	mail_sequence (stream,sequence))) return NIL;
 				/* got file? */  
-  if ((fd = open (mailboxfile (file,mailbox),O_BINARY|O_RDWR|O_CREAT,
+  if ((fd = open (dummy_file (file,mailbox),O_BINARY|O_RDWR|O_CREAT,
 		  S_IREAD|S_IWRITE)) < 0) {
     sprintf (LOCAL->buf,"Unable to open copy mailbox: %.80s",strerror (errno));
     mm_log (LOCAL->buf,ERROR);
@@ -726,7 +815,7 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
   }
   mm_critical (stream);		/* go critical */
 				/* get exclusive parse/append permission */
-  if ((ld = lockname (lock,mailbox,LOCK_EX)) < 0) {
+  if ((ld = lockname (lock,file,LOCK_EX)) < 0) {
     mm_log ("Unable to lock copy mailbox",ERROR);
     mm_nocritical (stream);
     return NIL;
@@ -752,8 +841,12 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
     mm_log (LOCAL->buf,ERROR);
     ftruncate (fd,sbuf.st_size);
   }
-  times.actime = sbuf.st_atime;	/* preserve atime and mtime */
-  times.modtime = sbuf.st_mtime;
+				/* set atime to now-1 if successful copy */
+  if (ret) times.actime = time (0) - 1;
+				/* else preserved \Marked status */
+  else times.actime = (sbuf.st_ctime > sbuf.st_atime) ?
+	 sbuf.st_atime : time (0);
+  times.modtime = sbuf.st_mtime;/* preserve mtime */
   utime (file,&times);		/* set the times */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
   close (fd);			/* close the file */
@@ -770,7 +863,7 @@ long tenex_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options)
       fsync (LOCAL->fd);
       fstat (LOCAL->fd,&sbuf);	/* get current write time */
       times.modtime = LOCAL->filetime = sbuf.st_mtime;
-      times.actime = time (0);		/* make sure atime remains greater */
+      times.actime = time (0);	/* make sure atime remains greater */
       utime (stream->mailbox,&times);
     }
   }
@@ -807,7 +900,7 @@ long tenex_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
 	((mailbox[2] == 'B') || (mailbox[2] == 'b')) &&
 	((mailbox[3] == 'O') || (mailbox[3] == 'o')) &&
 	((mailbox[4] == 'X') || (mailbox[4] == 'x')) && !mailbox[5])
-      dummy_create (NIL,"mail.txt");
+      dummy_create_path (NIL,"INBOX",NIL);
     else {
       mm_notify (stream,"[TRYCREATE] Must create mailbox before append",NIL);
       return NIL;
@@ -828,14 +921,14 @@ long tenex_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   if (!(*af) (stream,data,&flags,&date,&message)) return NIL;
 
 				/* open destination mailbox */
-  if (((fd = open(mailboxfile(file,mailbox),O_BINARY|O_WRONLY|O_APPEND|O_CREAT,
+  if (((fd = open(dummy_file(file,mailbox),O_BINARY|O_WRONLY|O_APPEND|O_CREAT,
 		   S_IREAD|S_IWRITE)) < 0) || !(df = fdopen (fd,"ab"))) {
     sprintf (tmp,"Can't open append mailbox: %s",strerror (errno));
     mm_log (tmp,ERROR);
     return NIL;
   }
 				/* get parse/append permission */
-  if ((ld = lockname (lock,mailbox,LOCK_EX)) < 0) {
+  if ((ld = lockname (lock,file,LOCK_EX)) < 0) {
     mm_log ("Unable to lock append mailbox",ERROR);
     close (fd);
     return NIL;
@@ -885,8 +978,11 @@ long tenex_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     mm_log (tmp,ERROR);
     ret = NIL;
   }
-  times.actime = sbuf.st_atime;	/* preserve atime and mtime */
-  times.modtime= sbuf.st_mtime;
+  if (ret) times.actime = time (0) - 1;
+				/* else preserved \Marked status */
+  else times.actime = (sbuf.st_ctime > sbuf.st_atime) ?
+	 sbuf.st_atime : time (0);
+  times.modtime = sbuf.st_mtime;/* preserve mtime */
   utime (file,&times);		/* set the times */
   fclose (df);			/* close the file */
   unlockfd (ld,lock);		/* release exclusive parse/append permission */
@@ -921,7 +1017,7 @@ long tenex_parse (MAILSTREAM *stream)
 {
   struct stat sbuf;
   MESSAGECACHE *elt = NIL;
-  char c,*s,*t,*x;
+  unsigned char c,*s,*t,*x;
   char tmp[MAILTMPLEN];
   unsigned long i,j;
   long curpos = LOCAL->filesize;
@@ -951,7 +1047,7 @@ long tenex_parse (MAILSTREAM *stream)
     LOCAL->buf[i] = '\0';	/* tie off buffer just in case */
     if (!(s = strchr (LOCAL->buf,'\012'))) {
       sprintf (tmp,"Unable to find newline at %lu in %lu bytes, text: %s",
-	       (unsigned long) curpos,i,LOCAL->buf);
+	       (unsigned long) curpos,i,(char *) LOCAL->buf);
       mm_log (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
@@ -960,7 +1056,7 @@ long tenex_parse (MAILSTREAM *stream)
     i = (s + 1) - LOCAL->buf;	/* note start of text offset */
     if (!((s = strchr (LOCAL->buf,',')) && (t = strchr (s+1,';')))) {
       sprintf (tmp,"Unable to parse internal header at %lu: %s",
-	       (unsigned long) curpos,LOCAL->buf);
+	       (unsigned long) curpos,(char *) LOCAL->buf);
       mm_log (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
@@ -981,7 +1077,7 @@ long tenex_parse (MAILSTREAM *stream)
     elt->private.msg.header.text.size = 0;
     x = s;			/* parse the header components */
     if (mail_parse_date (elt,LOCAL->buf) &&
-	(elt->private.msg.full.text.size = strtoul (s,&s,10)) &&
+	(elt->private.msg.full.text.size = strtoul (s,(char **) &s,10)) &&
 	(!(s && *s)) && isdigit (t[0]) && isdigit (t[1]) && isdigit (t[2]) &&
 	isdigit (t[3]) && isdigit (t[4]) && isdigit (t[5]) &&
 	isdigit (t[6]) && isdigit (t[7]) && isdigit (t[8]) &&
@@ -989,7 +1085,7 @@ long tenex_parse (MAILSTREAM *stream)
       elt->private.special.text.size = i;
     else {			/* oops */
       sprintf (tmp,"Unable to parse internal header elements at %ld: %s,%s;%s",
-	       curpos,LOCAL->buf,x,t);
+	       curpos,(char *) LOCAL->buf,(char *) x,(char *) t);
       mm_log (tmp,ERROR);
       tenex_close (stream,NIL);
       return NIL;
@@ -1028,7 +1124,7 @@ long tenex_parse (MAILSTREAM *stream)
   LOCAL->filesize = sbuf.st_size;
   fstat (LOCAL->fd,&sbuf);	/* get status again to ensure time is right */
   LOCAL->filetime = sbuf.st_mtime;
-  if (added) {			/* make sure atime updated */
+  if (added && !stream->rdonly){/* make sure atime updated */
     struct utimbuf times;
     times.actime = time (0);
     times.modtime = LOCAL->filetime;
@@ -1135,7 +1231,7 @@ void tenex_update_status (MAILSTREAM *stream,unsigned long msgno,long syncflag)
       fsync (LOCAL->fd);
       fstat (LOCAL->fd,&sbuf);	/* get new write time */
       times.modtime = LOCAL->filetime = sbuf.st_mtime;
-      times.actime = time (0);		/* make sure read is later */
+      times.actime = time (0);	/* make sure read is later */
       utime (stream->mailbox,&times);
     }
   }
