@@ -10,10 +10,10 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	9 May 1991
- * Last Edited:	10 May 2002
+ * Last Edited:	2 February 2004
  * 
  * The IMAP toolkit provided in this Distribution is
- * Copyright 2002 University of Washington.
+ * Copyright 1988-2004 University of Washington.
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this Distribution.
  */
@@ -31,6 +31,23 @@ extern int errno;		/* just in case */
 #include "misc.h"
 #include "mx.h"			/* highly unfortunate */
 
+/* Function prototypes */
+
+DRIVER *dummy_valid (char *name);
+void *dummy_parameters (long function,void *value);
+void dummy_list_work (MAILSTREAM *stream,char *dir,char *pat,char *contents,
+		      long level);
+long dummy_listed (MAILSTREAM *stream,char delimiter,char *name,
+		   long attributes,char *contents);
+long dummy_subscribe (MAILSTREAM *stream,char *mailbox);
+MAILSTREAM *dummy_open (MAILSTREAM *stream);
+void dummy_close (MAILSTREAM *stream,long options);
+long dummy_ping (MAILSTREAM *stream);
+void dummy_check (MAILSTREAM *stream);
+void dummy_expunge (MAILSTREAM *stream);
+long dummy_copy (MAILSTREAM *stream,char *sequence,char *mailbox,long options);
+long dummy_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data);
+
 /* Dummy routines */
 
 
@@ -212,7 +229,7 @@ long dummy_subscribe (MAILSTREAM *stream,char *mailbox)
       ((sbuf.st_mode & S_IFMT) == S_IFREG)
 #endif
       ) return sm_subscribe (mailbox);
-  sprintf (tmp,"Can't subscribe %s: not a mailbox",mailbox);
+  sprintf (tmp,"Can't subscribe %.80s: not a mailbox",mailbox);
   MM_LOG (tmp,ERROR);
   return NIL;
 }
@@ -342,13 +359,14 @@ long dummy_listed (MAILSTREAM *stream,char delimiter,char *name,
   DRIVER *d = NIL;
   unsigned long csiz;
   struct stat sbuf;
-  char tmp[MAILTMPLEN];
+  char *s,tmp[MAILTMPLEN];
 				/* don't \NoSelect dir if it has a driver */
   if ((attributes & LATT_NOSELECT) && (d = mail_valid (NIL,name,NIL)) &&
       (d != &dummydriver)) attributes &= ~LATT_NOSELECT;
   if (!contents ||		/* notify main program */
       (!(attributes & LATT_NOSELECT) && (csiz = strlen (contents)) &&
-       !stat (dummy_file (tmp,name),&sbuf) && (csiz <= sbuf.st_size) &&
+       (s = dummy_file (tmp,name)) && !stat (s,&sbuf) &&
+       (csiz <= sbuf.st_size) &&
        SAFE_SCAN_CONTENTS (d,tmp,contents,csiz,sbuf.st_size)))
     mm_list (stream,delimiter,name,attributes);
   return T;
@@ -366,7 +384,7 @@ long dummy_create (MAILSTREAM *stream,char *mailbox)
   long ret = NIL;
 				/* validate name */
   if (!(compare_cstring (mailbox,"INBOX") && (s = dummy_file (tmp,mailbox)))) {
-    sprintf (tmp,"Can't create %s: invalid name",mailbox);
+    sprintf (tmp,"Can't create %.80s: invalid name",mailbox);
     MM_LOG (tmp,ERROR);
   }
 				/* create the name, done if made directory */
@@ -412,7 +430,7 @@ long dummy_create_path (MAILSTREAM *stream,char *path,long dirmode)
 		       (int) mail_parameters(NIL,GET_MBXPROTECTION,NIL))) >= 0)
     ret = !close (fd);
   if (!ret) {			/* error? */
-    sprintf (tmp,"Can't create mailbox node %s: %s",path,strerror (errno));
+    sprintf (tmp,"Can't create mailbox node %.80s: %.80s",path,strerror (errno));
     MM_LOG (tmp,ERROR);
   }
   umask (mask);			/* restore mask */
@@ -429,11 +447,15 @@ long dummy_delete (MAILSTREAM *stream,char *mailbox)
 {
   struct stat sbuf;
   char *s,tmp[MAILTMPLEN];
+  if (!(s = dummy_file (tmp,mailbox))) {
+    sprintf (tmp,"Can't delete - invalid name: %.80s",s);
+    MM_LOG (tmp,ERROR);
+  }
 				/* no trailing / (workaround BSD kernel bug) */
-  if ((s = strrchr (dummy_file (tmp,mailbox),'/')) && !s[1]) *s = '\0';
+  if ((s = strrchr (tmp,'/')) && !s[1]) *s = '\0';
   if (stat (tmp,&sbuf) || ((sbuf.st_mode & S_IFMT) == S_IFDIR) ?
       rmdir (tmp) : unlink (tmp)) {
-    sprintf (tmp,"Can't delete mailbox %s: %s",mailbox,strerror (errno));
+    sprintf (tmp,"Can't delete mailbox %.80s: %.80s",mailbox,strerror (errno));
     MM_LOG (tmp,ERROR);
     return NIL;
   }
@@ -450,10 +472,11 @@ long dummy_delete (MAILSTREAM *stream,char *mailbox)
 long dummy_rename (MAILSTREAM *stream,char *old,char *newname)
 {
   struct stat sbuf;
-  char c,*s,tmp[MAILTMPLEN],mbx[MAILTMPLEN];
+  char c,*s,tmp[MAILTMPLEN],mbx[MAILTMPLEN],oldname[MAILTMPLEN];
 				/* no trailing / allowed */
-  if (!(s = dummy_file (mbx,newname)) || ((s = strrchr (s,'/')) && !s[1])) {
-    sprintf (mbx,"Can't rename %s to %s: invalid name",old,newname);
+  if (!dummy_file (oldname,old) || !(s = dummy_file (mbx,newname)) ||
+      ((s = strrchr (s,'/')) && !s[1])) {
+    sprintf (mbx,"Can't rename %.80s to %.80s: invalid name",old,newname);
     MM_LOG (mbx,ERROR);
     return NIL;
   }
@@ -466,10 +489,10 @@ long dummy_rename (MAILSTREAM *stream,char *old,char *newname)
     *s = c;			/* restore full name */
   }
 				/* rename of non-ex INBOX creates dest */
-  if (!compare_cstring (old,"INBOX") && stat (dummy_file (tmp,old),&sbuf))
+  if (!compare_cstring (old,"INBOX") && stat (oldname,&sbuf))
     return dummy_create (NIL,mbx);
-  if (rename (dummy_file (tmp,old),mbx)) {
-    sprintf (tmp,"Can't rename mailbox %s to %s: %s",old,newname,
+  if (rename (oldname,mbx)) {
+    sprintf (tmp,"Can't rename mailbox %.80s to %.80s: %.80s",old,newname,
 	     strerror (errno));
     MM_LOG (tmp,ERROR);
     return NIL;
@@ -491,18 +514,21 @@ MAILSTREAM *dummy_open (MAILSTREAM *stream)
   if (!stream) return &dummyproto;
   err[0] = '\0';		/* no error message yet */
 				/* can we open the file? */
-  if ((fd = open (dummy_file (tmp,stream->mailbox),O_RDONLY,NIL)) < 0) {
+  if (!dummy_file (tmp,stream->mailbox))
+    sprintf (err,"Can't open this name: %.80s",stream->mailbox);
+  else if ((fd = open (tmp,O_RDONLY,NIL)) < 0) {
 				/* no, error unless INBOX */
     if (compare_cstring (stream->mailbox,"INBOX"))
-      sprintf (err,"%s: %s",strerror (errno),stream->mailbox);
+      sprintf (err,"%.80s: %.80s",strerror (errno),stream->mailbox);
   }
   else {			/* file had better be empty then */
     fstat (fd,&sbuf);		/* sniff at its size */
     close (fd);
     if ((sbuf.st_mode & S_IFMT) != S_IFREG)
-      sprintf (err,"Can't open %s: not a selectable mailbox",stream->mailbox);
+      sprintf (err,"Can't open %.80s: not a selectable mailbox",
+	       stream->mailbox);
     else if (sbuf.st_size)	/* bogus format if non-empty */
-      sprintf (err,"Can't open %s (file %s): not in valid mailbox format",
+      sprintf (err,"Can't open %.80s (file %.80s): not in valid mailbox format",
 	       stream->mailbox,tmp);
   }
   if (err[0]) {			/* if an error happened */
@@ -512,7 +538,7 @@ MAILSTREAM *dummy_open (MAILSTREAM *stream)
   else if (!stream->silent) {	/* only if silence not requested */
     mail_exists (stream,0);	/* say there are 0 messages */
     mail_recent (stream,0);	/* and certainly no recent ones! */
-    stream->uid_validity = 1;
+    stream->uid_validity = time (0);
   }
   stream->inbox = T;		/* note that it's an INBOX */
   return stream;		/* return success */
@@ -536,24 +562,30 @@ void dummy_close (MAILSTREAM *stream,long options)
 
 long dummy_ping (MAILSTREAM *stream)
 {
+  MAILSTREAM *test;
 				/* time to do another test? */
-  if (time (0) >= (stream->gensym + 30)) {
-    MAILSTREAM *test = mail_open (NIL,stream->mailbox,OP_PROTOTYPE);
-    if (!test) return NIL;	/* can't get a prototype?? */
-    if (test->dtb == stream->dtb) {
-      stream->gensym = time (0);/* still hasn't changed */
-      return T;			/* try again later */
-    }
-				/* looks like a new driver? */
-    if (!(test = mail_open (NIL,stream->mailbox,NIL))) return NIL;
-    mail_close ((MAILSTREAM *)	/* flush resources used by dummy stream */
-		memcpy (fs_get (sizeof (MAILSTREAM)),stream,
-			sizeof (MAILSTREAM)));
+  if (time (0) >= ((time_t) (stream->gensym + 30))) {
+				/* has mailbox format changed? */
+    if ((test = mail_open (NIL,stream->mailbox,OP_PROTOTYPE)) &&
+	(test->dtb != stream->dtb) &&
+	(test = mail_open (NIL,stream->mailbox,NIL))) {
+				/* preserve some resources */
+      test->original_mailbox = stream->original_mailbox;
+      stream->original_mailbox = NIL;
+      test->sparep = stream->sparep;
+      stream->sparep = NIL;
+      test->sequence = stream->sequence;
+      mail_close ((MAILSTREAM *) /* flush resources used by dummy stream */
+		  memcpy (fs_get (sizeof (MAILSTREAM)),stream,
+			  sizeof (MAILSTREAM)));
 				/* swap the streams */
-    memcpy (stream,test,sizeof (MAILSTREAM));
-    fs_give ((void **) &test);	/* flush test now that copied */
+      memcpy (stream,test,sizeof (MAILSTREAM));
+      fs_give ((void **) &test);/* flush test now that copied */
 				/* make sure application knows */
-    mail_exists (stream,stream->recent = stream->nmsgs);
+      mail_exists (stream,stream->recent = stream->nmsgs);
+    }
+				/* still hasn't changed */
+    else stream->gensym = time (0);
   }
   return T;
 }
@@ -610,11 +642,11 @@ long dummy_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
   int e;
   char tmp[MAILTMPLEN];
   MAILSTREAM *ts = default_proto (T);
-  if (compare_cstring (mailbox,"INBOX") &&
-      ((fd = open (dummy_file (tmp,mailbox),O_RDONLY,NIL)) < 0)) {
+  if (compare_cstring (mailbox,"INBOX") && dummy_file (tmp,mailbox) &&
+      ((fd = open (tmp,O_RDONLY,NIL)) < 0)) {
     if ((e = errno) == ENOENT)	/* failed, was it no such file? */
       MM_NOTIFY (stream,"[TRYCREATE] Must create mailbox before append",NIL);
-    sprintf (tmp,"%s: %s",strerror (e),mailbox);
+    sprintf (tmp,"%.80s: %.80s",strerror (e),mailbox);
     MM_LOG (tmp,ERROR);		/* pass up error */
     return NIL;			/* always fails */
   }
@@ -624,7 +656,7 @@ long dummy_append (MAILSTREAM *stream,char *mailbox,append_t af,void *data)
     if (sbuf.st_size) ts = NIL;	/* non-empty file? */
   }
   if (ts) return (*ts->dtb->append) (stream,mailbox,af,data);
-  sprintf (tmp,"Indeterminate mailbox format: %s",mailbox);
+  sprintf (tmp,"Indeterminate mailbox format: %.80s",mailbox);
   MM_LOG (tmp,ERROR);
   return NIL;
 }
