@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: attach.c,v 4.66 2002/12/04 01:15:07 hubert Exp $";
+static char rcsid[] = "$Id: attach.c,v 4.69 2004/05/07 21:35:09 jpf Exp $";
 #endif
 /*
  * Program:	Routines to support attachments in the Pine composer 
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: attach.c,v 4.66 2002/12/04 01:15:07 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2000 by the University of Washington.
+ * 1989-2004 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -190,9 +190,11 @@ char *fn, *sz, *cmnt;
 	    *bfn = '\0';
 	    if(*fn == '\0' || !isdir(fn, NULL, NULL))
 	      strcpy(fn, (gmode & MDCURDIR)
-			    ? "."
-			    : ((gmode & MDTREE) || opertree[0])
-				 ? opertree : gethomedir(NULL));
+		     ? (browse_dir[0] ? browse_dir : ".")
+		     : ((gmode & MDTREE) || opertree[0])
+		     ? opertree 
+		     : (browse_dir[0] ? browse_dir
+			: gethomedir(NULL)));
 	    if((fbrv = FileBrowse(fn, NLINE, bfn, NLINE, sz, FB_READ | FB_ATTACH)) == 1){
 	      if ((strlen(fn)+strlen(S_FILESEP)+strlen(bfn)) < NLINE){
 		strcat(fn, S_FILESEP);
@@ -1203,20 +1205,33 @@ int   i;
  * prettysz - return pointer to string containing nice
  */
 char *prettysz(l)
-off_t l;
+    off_t l;
 {
     static char b[32];
+    long sz, left, right;
+
+    sz = (long) l;
+
+    if(sz < 1000L){
+	sprintf(b, "%ld B", sz);				/* xxx B */
+    }
+    else if(sz < 9950L){
+	left = (sz + 50L) / 1000L;
+	right = ((sz + 50L) - left * 1000L) / 100L;
+	sprintf(b, "%ld.%ld KB", left, right);			/* x.x KB */
+    }
+    else if(sz < 999500L){
+	sprintf(b, "%ld KB", (sz + 500L) / 1000L);		/* xxx KB */
+    }
+    else if(sz <  9950000L){
+	left = (sz + 50000L) / 1000000L;
+	right = ((sz + 50000L) - left * 1000000L) / 100000L;
+	sprintf(b, "%ld.%ld MB", left, right);			/* x.x MB */
+    }
+    else{
+	sprintf(b, "%ld MB", (sz + 500000L) / 1000000L);	/* xxx MB */
+    }
 
-    if(l < 1000)
-      sprintf(b, "%ld  B", (long)l);		/* xxx B */
-    else if(l < 10000)
-      sprintf(b, "%1.1f KB", (float)l/1000);    /* x.x KB */
-    else if(l < 1000000)
-      sprintf(b, "%ld KB", (long)l/1000L);	/* xxx KB */
-    else if(l < 10000000)
-      sprintf(b, "%1.1f MB", (float)l/1000000); /* x.x MB */
-    else
-      sprintf(b, "%ld MB", (long)l/1000000L);	/* xxx MB */
     return(b);
 }
 
