@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: browse.c,v 4.87 2003/04/23 19:41:10 hubert Exp $";
+static char rcsid[] = "$Id: browse.c,v 4.89 2004/05/07 21:36:40 jpf Exp $";
 #endif
 /*
  * Program:	Routines to support file browser in pico and Pine composer
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: browse.c,v 4.87 2003/04/23 19:41:10 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2003 by the University of Washington.
+ * 1989-2004 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -1740,7 +1740,10 @@ getfcells(dname, fb_flags)
 
     percdircells(mp);
     layoutcells(mp);
-    p_chdir(mp);
+    if(strlen(mp->dname) < NLINE)
+      strcpy(browse_dir, mp->dname);
+    else
+      browse_dir[0] = '\0';
     return(mp);
 }
 
