@@ -1,5 +1,5 @@
 /*
- * $Id: efunc.h,v 4.92 2003/04/09 18:34:31 hubert Exp $
+ * $Id: efunc.h,v 4.95 2004/05/07 21:41:10 jpf Exp $
  *
  * Program:	Pine's composer and pico's function declarations
  *
@@ -20,7 +20,7 @@
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2002 by the University of Washington.
+ * 1989-2004 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -226,9 +226,7 @@ extern	char *pfnexpand PROTO((char *, size_t));
 extern	int compresspath PROTO((char *, char *, int));
 extern	void tmpname PROTO((char *, char *));
 extern  char *temp_nam PROTO((char *, char *));
-#if defined(DOS) || defined(OS2)
 extern  char *temp_nam_ext PROTO((char *, char *, char *));
-#endif
 extern	void makename PROTO((char *, char *));
 extern	int copy PROTO((char *, char *));
 extern	int ffwopen PROTO((char *, int));
@@ -335,6 +333,7 @@ extern	int wrapword PROTO((void));
 extern	int backword PROTO((int, int));
 extern	int forwword PROTO((int, int));
 extern	int fillpara PROTO((int, int));
+extern	int fillbuf PROTO((int, int));
 extern	int inword PROTO((void));
 extern	int quote_match PROTO((char *, LINE *, char *, int));
 
