@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: search.c,v 4.29 2002/12/04 01:15:07 hubert Exp $";
+static char rcsid[] = "$Id: search.c,v 4.31 2004/05/07 21:45:04 jpf Exp $";
 #endif
 /*
  * Program:	Searching routines
@@ -21,7 +21,7 @@ static char rcsid[] = "$Id: search.c,v 4.29 2002/12/04 01:15:07 hubert Exp $";
  * permission of the University of Washington.
  * 
  * Pine, Pico, and Pilot software and its included text are Copyright
- * 1989-2002 by the University of Washington.
+ * 1989-2004 by the University of Washington.
  * 
  * The full text of our legal notices is contained in the file called
  * CPYRIGHT, included with this distribution.
@@ -230,6 +230,11 @@ forwsearch(f, n)
 	    mlerase();
 	    FWS_RETURN(TRUE);
 
+	  case (CTRL|'J'):
+	    fillbuf(0, 1);
+	    mlerase();
+	    FWS_RETURN(TRUE);
+
 	  case  (CTRL|'R'):        /* toggle replacement option */
 	    repl_mode = !repl_mode;
 	    break;
@@ -494,7 +499,7 @@ int   repl_mode;
 	register int s;
 	int	     i = 0;
 	char	     tpat[NPAT+20];
-	EXTRAKEYS    menu_pat[7];
+	EXTRAKEYS    menu_pat[8];
 
 	menu_pat[i = 0].name = "^Y";
 	menu_pat[i].label    = "FirstLine";
@@ -526,6 +531,11 @@ int   repl_mode;
 	    menu_pat[i].label  = "End of Para";
 	    menu_pat[i].key    = (CTRL|'O');
 	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	    menu_pat[++i].name = "^J";
+	    menu_pat[i].label  = "FullJustify";
+	    menu_pat[i].key    = (CTRL|'J');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
 	}
 
 	menu_pat[++i].name = NULL;
@@ -620,7 +630,7 @@ int   text_mode;
 	register int s;
 	int	     i;
 	char	     tpat[NPAT+20];
-	EXTRAKEYS    menu_pat[6];
+	EXTRAKEYS    menu_pat[7];
 
 	menu_pat[i = 0].name = "^Y";
 	menu_pat[i].label    = "FirstLine";
@@ -647,6 +657,11 @@ int   text_mode;
 	    menu_pat[i].label  = "End of Para";
 	    menu_pat[i].key    = (CTRL|'O');
 	    KS_OSDATASET(&menu_pat[i], KS_NONE);
+
+	    menu_pat[++i].name = "^J";
+	    menu_pat[i].label  = "FullJustify";
+	    menu_pat[i].key    = (CTRL|'J');
+	    KS_OSDATASET(&menu_pat[i], KS_NONE);
 	}
 
 	menu_pat[++i].name = NULL;
