@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: args.c,v 4.120 2003/04/28 22:51:16 hubert Exp $";
+static char rcsid[] = "$Id: args.c,v 4.123 2003/10/10 21:05:16 jpf Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -138,6 +138,7 @@ char *args_pine_args[] = {
 #ifdef PASSFILE
 " -passfile <fully_qualified_filename>\tSet the password file to something other",
 "\t\tthan the default",
+" -nowrite_passfile\tRead from a passfile if there is one, but never offer to write a password to the passfile",
 #endif /* PASSFILE */
 " -x <config>\tUse configuration exceptions in <config>.",
 "\t\tExceptions are used to override your default pinerc",
@@ -160,13 +161,15 @@ char *args_pine_args[] = {
 " -convert_sigs -p <pinerc>   convert signatures to literal signatures",
 #if	defined(_WINDOWS)
 " -install \tPrompt for some basic setup information",
-" -registry <cmd>\tWhere cmd is set,clear,clearsilent,dump",
+" -registry <cmd>\tWhere cmd is set,noset,clear,clearsilent,dump",
 #endif
 " -<option>=<value>   Assign <value> to the pinerc option <option>",
 "\t\t     e.g. -signature-file=sig1",
 "\t\t     e.g. -color-style=no-color",
 "\t\t     e.g. -feature-list=enable-sigdashes",
-"\t\t     (Note: feature-list is additive)",
+"\t\t     Note: feature-list is additive.",
+"\t\t     You may leave off the \"feature-list=\" part of that,",
+"\t\t     e.g. -enable-sigdashes",
 NULL
 };
 
@@ -306,6 +309,10 @@ Loop: while(--ac > 0)
 
 		  goto Loop;
 	      }
+	      else if(strcmp(*av, "nowrite_passfile") == 0){
+		  ps_global->nowrite_passfile = 1;
+		  goto Loop;
+	      }
 #endif  /* PASSFILE */
 	      else if(strcmp(*av, "create_lu") == 0){
 		  if(ac > 2){
@@ -491,7 +498,10 @@ Loop: while(--ac > 0)
 	      else if(strcmp(*av, "registry") == 0){
 		  if(--ac){
 		      if(!strucmp(*++av, "set")){
-			  pine_state->update_registry = 1;
+			  pine_state->update_registry = UREG_ALWAYS_SET;
+		      }
+		      else if(!strucmp(*av, "noset")){
+			  pine_state->update_registry = UREG_NEVER_SET;
 		      }
 		      else if(!strucmp(*av, "clear")){
 			  if(!mswin_reg(MSWR_OP_BLAST, MSWR_NULL, NULL, 0))
