@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: help.c,v 4.129 2003/08/29 21:28:22 jpf Exp $";
+static char rcsid[] = "$Id: help.c,v 4.134 2004/04/08 16:58:52 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: help.c,v 4.129 2003/08/29 21:28:22 jpf Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2002 by the University of Washington.
+   1989-2004 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -221,13 +221,13 @@ helper_internal(text, otext, frag, title, flags)
 	    if(!struncmp(shown_text[0], "<html>", 6))
 	      gf_link_filter(gf_html2plain,
 			     gf_html2plain_opt("x-pine-help:",
-					ps_global->ttyo->screen_cols,
-					&handles, GFHP_LOCAL_HANDLES));
+					       ps_global->ttyo->screen_cols, NULL,
+					       &handles, GFHP_LOCAL_HANDLES));
 	    else
 	      gf_link_filter(gf_wrap, gf_wrap_filter_opt(
 						  ps_global->ttyo->screen_cols,
 						  ps_global->ttyo->screen_cols,
-						  0, GFW_HANDLES));
+						  NULL, 0, GFW_HANDLES));
 
 	    error = gf_pipe(helper_getc, pc);
 
@@ -353,7 +353,8 @@ helper_internal(text, otext, frag, title, flags)
 		    clrbitn(HLP_NEXT_HANDLE, sargs.keys.bitmap);
 		}
 
-		if(text != main_menu_tx) /* only main can "print all" */
+		if(text != main_menu_tx
+		   && text != h_mainhelp_pinehelp)
 		  clrbitn(HLP_ALL_KEY, sargs.keys.bitmap);
 
 		if(frag){
@@ -530,7 +531,8 @@ print_help(text)
 	char *p;
 
 	gf_link_filter(gf_html2plain,
-		       gf_html2plain_opt(NULL,80,NULL,GFHP_STRIPPED));
+		       gf_html2plain_opt(NULL,80,NULL,
+					 NULL,GFHP_STRIPPED));
 	for(i = 1; i <= 5 && text[i]; i++)
 	  if(!struncmp(text[i], "<title>", 7)
 	     && (p = srchstr(text[i] + 7, "</title>"))
@@ -541,7 +543,7 @@ print_help(text)
 	  }
     }
     else
-      gf_link_filter(gf_wrap, gf_wrap_filter_opt(80, 80, 0, 0));
+      gf_link_filter(gf_wrap, gf_wrap_filter_opt(80, 80, NULL, 0, 0));
 
     gf_link_filter(gf_line_test,
 		   gf_line_test_opt(print_help_page_break, NULL));
@@ -1056,7 +1058,7 @@ review_messages()
 	gf_link_filter(gf_wrap,
 		       gf_wrap_filter_opt(ps_global->ttyo->screen_cols - 4,
 					  ps_global->ttyo->screen_cols,
-					  INDENT, 0));
+					  NULL, INDENT, 0));
 	gf_set_so_readc(&gc, in_store);
 	gf_set_so_writec(&pc, out_store);
 	gf_pipe(gc, pc);
@@ -1589,10 +1591,10 @@ gripe_newbody(ps, body, msgno, flags)
 					 NULL, NULL,
 					 PIPE_READ | PIPE_STDERR | PIPE_USER,
 					 0)){
-		    gf_set_readc(&gc, (void *)syspipe->in.f, 0, FileStar);
+		    gf_set_readc(&gc, (void *)syspipe, 0, PipeStar);
 		    gf_filter_init();
 		    error = gf_pipe(gc, pc);
-		    (void) close_system_pipe(&syspipe);
+		    (void) close_system_pipe(&syspipe, NULL, 0);
 		}
 		else
 		  error = "executing config collector";
