@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: status.c,v 4.127 2002/11/27 23:22:12 hubert Exp $";
+static char rcsid[] = "$Id: status.c,v 4.134 2004/05/07 22:56:45 jpf Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -22,7 +22,7 @@ static char rcsid[] = "$Id: status.c,v 4.127 2002/11/27 23:22:12 hubert Exp $";
    permission of the University of Washington.
 
    Pine, Pico, and Pilot software and its included text are Copyright
-   1989-2002 by the University of Washington.
+   1989-2004 by the University of Washington.
 
    The full text of our legal notices is contained in the file called
    CPYRIGHT, included with this distribution.
@@ -192,7 +192,8 @@ q_status_message(flags, min_time, max_time, message)
     else
       message_queue = new->next = new->prev = new;
 
-    dprint(9, (debugfile, "q_status_message(%s)\n", clean_msg));
+    dprint(9, (debugfile, "q_status_message(%s)\n",
+	   clean_msg ? clean_msg : "?"));
 }
 
 
@@ -314,6 +315,19 @@ q_status_message5(flags, min_t, max_t, s, a1, a2, a3, a4, a5)
 }
 
 
+/*VARARGS1*/
+void
+q_status_message6(flags, min_t, max_t, s, a1, a2, a3, a4, a5, a6)
+    int   flags;
+    int   min_t, max_t;
+    char *s;
+    void *a1, *a2, *a3, *a4, *a5, *a6;
+{
+    sprintf(tmp_20k_buf, s, a1, a2, a3, a4, a5, a6);
+    q_status_message(flags, min_t, max_t, tmp_20k_buf);
+}
+
+
 /*----------------------------------------------------------------------
         Put a message with 7 printf argument on queue for status line
 
@@ -589,8 +603,28 @@ display_message(command)
 	  displayed_time = time(0);
 	  output_message(message_queue);
       }
-      else
-	d_q_status_message();
+      else{
+	  if(message_queue->text){
+	      char   buf[1000];
+	      char  *append = " [not actually shown]";
+	      char  *ptr;
+	      size_t len;
+
+	      len = strlen(message_queue->text) + strlen(append);
+	      if(len < sizeof(buf))
+		ptr = buf;
+	      else
+		ptr = (char *) fs_get((len+1) * sizeof(char));
+
+	      strcpy(ptr, message_queue->text);
+	      strcat(ptr, append);
+	      add_review_message(ptr, -1);
+	      if(ptr != buf)
+		fs_give((void **) &ptr);
+	  }
+
+	  d_q_status_message();
+      }
 
     needs_clearing = 0;				/* always cleared or written */
     dprint(9, (debugfile,
@@ -674,7 +708,7 @@ d_q_status_message()
 {
     if(message_queue){
 	dprint(9, (debugfile, "d_q_status_message(%.40s)\n",
-		   message_queue->text));
+	       message_queue->text ? message_queue->text : "?"));
 	if(!LAST_MESSAGE(message_queue)){
 	    SMQ_T *p = message_queue;
 	    p->next->prev = p->prev;
@@ -865,7 +899,8 @@ output_message(mq_entry)
 {
     int rv = 0;
 
-    dprint(9, (debugfile, "output_message(%s)\n", mq_entry->text));
+    dprint(9, (debugfile, "output_message(%s)\n",
+	   (mq_entry && mq_entry->text) ? mq_entry->text : "?"));
 
     if((mq_entry->flags & SM_DING) && F_OFF(F_QUELL_BEEPS, ps_global))
       Writechar(BELL, 0);			/* ring bell */
@@ -1483,6 +1518,16 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
           case ctrl('M'):
           case ctrl('J'):
             ch = dflt;
+	    for(i = 0; esc_list && esc_list[i].ch != -1; i++)
+	      if(ch == esc_list[i].rval){
+		  int len, n;
+
+		  MoveCursor(real_line, len=min(RAD_BUT_COL+strlen(q),maxcol+1));
+		  for(n = 0, len = ps_global->ttyo->screen_cols - len;
+		      esc_list[i].label[n] && len > 0; n++, len--)
+		    Writechar(esc_list[i].label[n], 0);
+		  break;
+	      }
             goto out_of_loop;
 
           case ctrl('C'):
@@ -1549,8 +1594,9 @@ radio_buttons(prompt, line, esc_list, dflt, on_ctrl_C, help_text, flags)
 	    if(flags & RB_NO_NEWMAIL)
 	      goto newcmd;
 
-	    i = new_mail(0, 2, NM_DEFER_SORT);
-	    if(ps_global->expunge_count && flags & RB_SEQ_SENSITIVE){
+	    i = new_mail(0, VeryBadTime, NM_DEFER_SORT);
+	    if(sp_expunge_count(ps_global->mail_stream)
+	       && flags & RB_SEQ_SENSITIVE){
 		if(on_ctrl_C)
 		  ch = on_ctrl_C;
 		else
